from __future__ import with_statement
import es
import time
import Queue
import threading
import psyco
psyco.full()

q = Queue.Queue() 
register_lock = threading.Lock()
tickreg = False;

class TimeSortedQueue(object):
  class TimeNode(object):
    def __init__(self, gotime, cmd, name):
      self.name = name
      self.gotime = float(gotime)
      self.cmd = cmd
    def __cmp__(self, b):
      #Should return a negative integer if self < other, zero if self == other, a positive integer if self > other
      return self.gotime - b.gotime   
  def __init__(self):
    self.nodes = []
    self.lock = threading.Lock()
  def addNode(self, node):
    with self.lock:
      self.nodes.append(node)
      self.nodes.sort()
  def add(self, gotime, cmd, args=(), kw=None, name=None):
    if not isinstance(args, tuple):
      args = (args,)
    self.addNode(self.TimeNode(gotime,(cmd, args, kw or {}),name))
  def getFirst(self):
    with self.lock:
      if self.nodes:
        return self.nodes[0]
      else:
        return None
  def getList(self):
    with self.lock:
      return self.nodes
  def removeByName(self, name):
    if not name: return   # can't kill items with no name
    with self.lock:
      self.nodes = filter(lambda i: i.name != name, self.nodes)
  def remove(self, item):
    with self.lock:
      if item in self.nodes:
        self.nodes.remove(item)
  def empty(self):
    with self.lock:
      return not bool(self.nodes)
  def printQueue(self):
    with self.lock:
      for i in self.nodes:
        es.dbgmsg(0, "[%8s] in %s seconds from now,\n  %s\n" % (i.name, i.gotime - time.time(), i.cmd)) 

timeq = TimeSortedQueue()


##################################
# begin public external interfaces
# begin public external interfaces

def delayed(seconds, cmd, args=(), kw=None):
    registerTicker()
    timeq.add(time.time()+seconds, cmd, args, kw)

def delayedname(seconds, name, cmd, args=(), kw=None):
    registerTicker()
    timeq.add(time.time()+seconds, cmd, args, kw, name)

def cancelDelayed(name):
    timeq.removeByName(name)
    
def listDelayed():
    timeq.printQueue()
    
def queue(function, a=(), kw=None):
    registerTicker()
    if not isinstance(a, tuple):
      a = (a,)
    q.put((function, a, kw or {}))

# end public external interfaces
# end public external interfaces
##################################


def registerTicker():
  global tickreg
  with register_lock:
    if not tickreg:
      tickreg = True
      es.addons.registerTickListener(tick)

def unregisterTicker():
  global tickreg
  with register_lock:
    if tickreg:
      tickreg = False
      es.addons.unregisterTickListener(tick)


def _executenode(node):
    function, a, kw = node.cmd
    try:
      #print "executing %s %f seconds too late. sched: %f now: %f" % (function.__name__, now-task.gotime, task.gotime, now)
      if function:
        function(*a, **kw)
    except AttributeError, att:
      print "gamethread delayeda error:", att
    except Exception, inst:
      print "gamethread delayed error:", inst
    timeq.remove(node)

# gameframe callback
def tick():
  # handle normal q
  while not q.empty():
    function, a, kw = q.get()
    try:
      function(*a, **kw)
    except Exception, inst:
      print "gamethread queue error:", inst

  # check the first one to speed this up since none are ready unless it is.
  first = timeq.getFirst()
  if first:
    now = time.time()
    if first.gotime <= now:
      _executenode(first)
      tasks = timeq.getList()
      for task in tasks:
        if task.gotime <= now:
          _executenode(task)
        else:
          break    

  if timeq.empty() and q.empty():  
    unregisterTicker()
        
