import es

types={}
types[0] = "TYPE_NONE"
types[1] = "TYPE_STRING"
types[2] = "TYPE_INT"
types[3] = "TYPE_FLOAT"
types[4] = "TYPE_PTR"
types[5] = "TYPE_WSTRING"
types[6] = "TYPE_COLOR"
types[7] = "TYPE_NUMTYPES"


def getKeyGroup(name):
    x = es.keygroupgetpointer(name)
    if x is not None and x != 0:
        k = KeyValues(existingid=x)
        k._parent_ = 0
        return k
    else:
        return None
        
class KeyValues(object):
    _id_ = 0
    _parent_ = None
    def __init__(self, name=None, parent=None, existingid=None, filename=None):
        if existingid is not None:
            #print "kv", existingid, parent
            self._id_ = existingid
            self._parent_ = parent
        else:
            if parent is None:
                self._id_ = es.keypcreate()
                if filename is not None:
                    self.load(filename)
                if name is not None:
                    es.keypsetname(self._id_, name)
            else:
                self._parent_ = parent
                if name is None:
                    self._id_ = es.keypcreatesubkey(parent._id_)
                else:
                    self._id_ = es.keypfindsubkey(parent._id_, name)
    def load(self, filename):
        es.keyploadfromfile(self._id_, filename)
    def save(self, filename):
        es.keypsavetofile(self._id_, filename)
    def getParent(self):
        return self._parent_
    def setParent(self, parent):
        self._parent_ = parent
    def keys(self):
        keyers = []
        for i in self:
            keyers.append(i.getName())
        return keyers
    def __del__(self):
        if self._id_ > 0 and self._parent_ is None:
            es.keypdelete(self._id_)
    def __str__(self):
        return es.keypgetname(self._id_)
    def __hash__(self):
        return self._id_
    def __contains__(self, item):
        if item in self.keys():
            return True
        else:
            return False
    def __getitem__(self, name):
        x = es.keypfindsubkey(self._id_, name, False)
        if x is None or x == 0:
            raise KeyError
        else:
            typ = es.keypgetdatatype(x)
            if types[typ] == "TYPE_NONE":
                return KeyValues(parent=self, existingid=x)                
            elif types[typ] == "TYPE_STRING" or types[typ] == "TYPE_WSTRING":
                return es.keypgetstring(self._id_, name)
            elif types[typ] == "TYPE_INT":
                return es.keypgetint(self._id_, name)
            elif types[typ] == "TYPE_FLOAT":
                return es.keypgetfloat(self._id_, name)
##          types[4] = "TYPE_PTR"
##          types[6] = "TYPE_COLOR"
##          types[7] = "TYPE_NUMTYPES"
            else:
                raise NotImplemented, "KeyValues does not yet support type: %s" % types[typ]
    def __setitem__(self, name, value):
        if (isinstance(value, KeyValues)):
            x = es.keypfindsubkey(self._id_, name, True)
            if x:
                es.keyprecursivekeycopy(x, value._id_)
        else:
            if (isinstance(value, int)):
                es.keypsetint(self._id_, name, value)
            elif (isinstance(value, float)):
                es.keypsetfloat(self._id_,name, value)
            else:
                es.keypsetstring(self._id_, name, str(value))
    def getName(self):
        return es.keypgetname(self._id_)
    def getString(self, var=""):
        return es.keypgetstring(self._id_, var)
    def getInt(self, var):
        return es.keypgetint(self._id_, var)
    def getFloat(self, var):
        return es.keypgetfloat(self._id_, var)
    def load(self, filename):
        es.keyploadfromfile(self._id_, filename)
    def save(self, filename):
        es.keypsavetofile(self._id_, filename)
    class KeyValuesIter(object):
        def __init__(self, currentkey):
            self.nextkeyid = currentkey._id_
            self.parent = currentkey
        def __iter__(self):
            return self
        def next(self):
            if self.nextkeyid == self.parent._id_:
                self.nextkeyid = es.keypgetfirstsubkey(self.nextkeyid)
            else:
                self.nextkeyid = es.keypgetnextkey(self.nextkeyid)
            if self.nextkeyid is None or self.nextkeyid == 0:
                raise StopIteration
            return KeyValues(existingid=self.nextkeyid, parent=self.parent)
    def __iter__(self):
        return self.KeyValuesIter(self)

