import es
from es import server_var

info = es.AddonInfo()
info.name     = "Python Unit Test Script"
info.version  = "1.0"
info.url      = "http://mattie.info/cs"
info.basename = "pyunittest"
info.author   = "Mattie" 

def load():
    whiletest()
    vectors()
    cmdtest()
    keytest()
    svartest()

    s = es.ServerVar("sv_gravity")
    s.set(795)
    if s != 795:
      print "svfailed5"
    if s != "795":
      print "svfailed6"
    print "__coerce__ worked fine"
    
def svartest():
    s = es.ServerVar("sv_gravity")
    text = str(s)
    if text != "800":
      print "svfailed1"
    val = int(s)
    if val != 800:
      print "svfailed2"
    fl = float(s)
    if fl != 800.0:
      print "svfailed3"
    else:
      print fl
    if s.getName() != "sv_gravity":
      print "svfailed4"
    s.set(795)
    if s != 795:
      print "svfailed5"
    if s != "795":
      print "svfailed6"
    else:
      print "coercion works for ServerVar()"
    print "svartest passed"


def whiletest():
    i = 5000
    es.set("myvar", 5000)
    print server_var['myvar']    
    while i:
        i-=1
        es.server.cmd("es_xmath myvar - 1")
    print server_var['myvar']
    
def vectors():
    y = es.createvectorstring(3.5,4.5,6.8)
    if y != "3.500000,4.500000,6.800000":
        print "failed",y
    else:
        print "success1",y
    z = es.splitvectorstring(y)
    if z[0] != 3.5 or z[1] != 4.5 or round(z[2],1) != 6.8:
        print "failed2",z
    else:
        print "success2",z

def cmdtest():
    cmdtest_register()
    es.server.cmd("pycommand passed")
    if server_var['cmdtest_pass'] == "passed":
        print "cmdtest_passed"
    else:
        print "cmdtest_failed"
    if server_var['cmdtest_pass2'] == "passed":
        print "cmdtest2_passed"
    else:
        print "cmdtest2_failed"
    
def cmdtest_register():
    es.regcmd("pycommand", "pyunittest/cmdtest_run", "Stuff")
    
def cmdtest_run():
    es.set("cmdtest_pass", es.getargv(1))
    print "cmdtest()"
    es.doblock("pyunittest/cmdtest_block")

def cmdtest_block():
    es.set("cmdtest_pass2", es.getargv(1))
    print "cmdtest2()"

def keytest():
    keytest_keygroup()
    keytest_newkey()
    
def keytest_newkey():
    x = es.keypcreate()
    es.keypsetname(x, "justice")
    keytest2(x)
    es.keypdelete(x)
    x = es.keypcreate()
    es.keypsetname(x, "justice2")
    keytest3(x)
    es.keypdelete(x)
    return

def keytest_keygroup():
    es.keygroupcreate("justice")
    x = es.keygroupgetpointer("justice")
    keytest2(x)
    es.keygroupdelete("justice2")
    es.keygroupcreate("justice2")
    x = es.keygroupgetpointer("justice2")
    keytest3(x)
    import keyvalues
    y = keyvalues.getKeyGroup("justice2")
    printTree(y)
    es.keygroupdelete("justice2")
    return
    
def printTree(k, depth=0):
    print " "*depth, "Name:", k.getName(), "id:", k._id_
    for j in k:
        printTree(j, depth+1)
        
def keytest3(x):
    es.keyploadfromfile(x, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")
    y = es.keypfindsubkey(x, "subkey", False)
    outcome = es.keypgetint(y, "hello")
    if outcome != 99:
      print "keytest() failed10e"
      return
    z = es.keypfindsubkey(y, "test", True)
    name = es.keypgetname(z)
    if name != "test":
      print "keytest() failed10d"
      return
    pp = es.keypcreate()
    es.keyploadfromfile(pp, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")
    es.keyprecursivekeycopy(z, pp)
    name = es.keypgetname(z)
    if name != "justice2":
      print "keytest() failed10f"
      return
    
def keytest2(x):
    if x <= 0:
      print "keytest() failed"
      return
    name = es.keypgetname(x)
    if name != "justice":
      print "keytest() failed2"
      return
    es.keypsetname(x, "justice2")
    name = es.keypgetname(x)
    if name != "justice2":
      print "keytest() failed3"
      return
    y = es.keypfindsubkey(x, "subkey", False)
    if y is not None:
      print "keytest() failed4 %d" % y
      return
    y = es.keypfindsubkey(x, "subkey", True)
    subname = es.keypgetname(y)
    if subname != "subkey":
      print "keytest() failed5"
      return
    r = es.keypfindsubkey(x, "subkey", True)
    if r != y:
      print "keytest() failed6"
      return
    q = es.keypfindsubkey(x, "subkey2", True)
    subname = es.keypgetname(q)
    if subname != "subkey2":
      print "keytest() failed7"
      return
    w = es.keypfindsubkey(x, "subkey3", True)
    e = es.keypfindsubkey(x, "subkey3", True)
    print "keylist"
    p = es.keypgetfirstsubkey(x)
    n = 0
    while p:
      print "Key:", es.keypgetname(p)
      n+=1
      p = es.keypgetnextkey(p)
    if n!=3:
      print "keytest() failed7a"
      return      
    if w != e:
      print "keytest() failed8"
      return
    es.keypdetachsubkey(x, w)
    e = es.keypfindsubkey(x, "subkey3", False)
    if e is not None:
      print "keytest() failed9"
      return
    es.keypdelete(w)
    p = es.keypgetfirstsubkey(x)
    n = 0
    while p:
      print "Key:", es.keypgetname(p)
      n+=1
      p = es.keypgetnextkey(p)
    if n!=2:
      print "keytest() failed9a"
      return
    if not es.keypisempty(r):
      print "keytest() failed9b"
      return
    # setstring    
    es.keypsetstring(r, "hello", "1.3")
    if es.keypisempty(r):
      print "keytest() failed9c"
      return
    outcome = es.keypgetstring(y, "hello")
    if outcome != "1.3":
      print "keytest() failed10a (%s)" % outcome
      return
    outcome = es.keypgetint(y, "hello")
    if outcome != 1:
      print "keytest() failed10b"
      return
    outcome = es.keypgetfloat(y, "hello")
    if outcome != 1.3:
      print "keytest() failed10c"
      return
    # setint
    es.keypsetint(y, "hello", 99)
    outcome = es.keypgetint(y, "hello")
    if outcome != 99:
      print "keytest() failed10d"
      return
    # setfloat
    es.keypsetint(y, "hello", 99)
    outcome = es.keypgetint(y, "hello")
    if outcome != 99:
      print "keytest() failed10d"
      return
    # save to file
    es.keypsavetofile(x, server_var['eventscripts_addondir'] + "/pyunittest/myfile.vdf")
    print "keytest() passed"

